@@ -3,35 +3,38 @@
  *  Module    : active.c
  *  Author    : I. Lea
  *  Created   : 1992-02-16
- *  Updated   : 2016-07-29
+ *  Updated   : 2023-12-22
  *  Notes     :
  *
- * Copyright (c) 1992-2016 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1992-2024 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
+ *
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote
- *    products derived from this software without specific prior written
- *    permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
- * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * 3. Neither the name of the copyright holder nor the names of its
+ *    contributors may be used to endorse or promote products derived from
+ *    this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 
@@ -41,6 +44,9 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
+#ifndef STPWATCH_H
+#	include "stpwatch.h"
+#endif /* !STPWATCH_H */
 
 /*
  * List of allowed separator chars in active file
@@ -92,7 +98,7 @@ need_reread_active_file(
 
 /*
  * Resync active file when reread_active_file_secs have passed or
- * force_reread_actve_file is set.
+ * force_reread_active_file is set.
  * Return TRUE if a reread was performed
  */
 t_bool
@@ -170,7 +176,7 @@ active_add(
 
 	if (moderated[0] == '/') {
 		ptr->type = GROUP_TYPE_SAVE;
-		ptr->spooldir = my_strdup(moderated); /* TODO: Unix'ism, other OSs need transformation */
+		ptr->spooldir = my_strdup(moderated);
 	} else {
 		ptr->type = GROUP_TYPE_NEWS;
 		ptr->spooldir = spooldir;		/* another global - sigh */
@@ -199,6 +205,12 @@ process_bogus(
 
 	active_add(ptr, T_ARTNUM_CONST(0), T_ARTNUM_CONST(1), T_ARTNUM_CONST(0), "n");
 	ptr->bogus = TRUE;		/* Mark it bogus */
+	/*
+	 * Set pointer to default attributes
+	 */
+	if (ptr->attribute && !ptr->attribute->global)
+		free(ptr->attribute);
+	ptr->attribute = scopes[0].attribute;
 
 	if (my_group_add(name, FALSE) < 0)
 		return TRUE;
@@ -218,40 +230,62 @@ parse_active_line(
 	char *moderated)
 {
 	char *p = NULL, *q = NULL, *r = NULL;
+#ifdef DEBUG
+	char *l;
+#endif /* DEBUG */
 	t_bool lineok = FALSE;
 
 	if (line[0] == '#' || line[0] == '\0')
-		return FALSE;
+		return lineok;
+
+#ifdef DEBUG
+	l = my_strdup(line);
+#endif /* DEBUG */
 
 	if (strtok(line, ACTIVE_SEP)) {		/* skip group name */
 		if ((p = strtok(NULL, ACTIVE_SEP))) {	/* group max count */
 			if ((q = strtok(NULL, ACTIVE_SEP))) {	/* group min count */
-				r = strtok(NULL, ACTIVE_SEP);	/* mod status or path to mailgroup */
-				lineok = TRUE;
+				if ((r = strtok(NULL, ACTIVE_SEP)))	/* mod status or path to mailgroup */
+					lineok = TRUE;
 			}
 		}
 	}
 
-	if (!p || !q || !r || !lineok) {
+	if (!lineok) {
 #ifdef DEBUG
-		if (debug & DEBUG_NNTP)
-			debug_print_file("NNTP", txt_bad_active_file, line);
+		/* TODO: This also logs broken non NNTP active lines (i.e. mail.active) to NNTP */
+		if ((debug & DEBUG_NNTP) && verbose > 1)
+			debug_print_file("NNTP", "Active file corrupt - %s", l);
+
 #endif /* DEBUG */
-		error_message(2, _(txt_bad_active_file), line);
-		return FALSE;
+		if (!p || !q) {
+#ifdef DEBUG
+			free(l);
+#endif /* DEBUG */
+			return lineok;
+		}
 	}
 
 	*max = atoartnum(p);
 	*min = atoartnum(q);
-	strcpy(moderated, r);
 
-	return TRUE;
+	if (!lineok) { /* missing moderation flag - seen on usenet.farm */
+		strcpy(moderated, "y");	/* guess posting is fine */
+		lineok = TRUE;
+	} else
+		strcpy(moderated, r);
+
+#ifdef DEBUG
+	free(l);
+#endif /* DEBUG */
+
+	return lineok;
 }
 
 
 #ifdef NNTP_ABLE
 /*
- * Parse line from "LIST COUNTS" (RFC 6048)
+ * Parse line from "LIST COUNTS" (RFC 6048 2.2.)
  * group high low count status
  */
 static t_bool
@@ -281,10 +315,9 @@ parse_count_line(
 
 	if (!p || !q || !r || !s || !lineok) {
 #	ifdef DEBUG
-		if (debug & DEBUG_NNTP)
-			debug_print_file("NNTP", _(txt_unparseable_counts), line);
+		if ((debug & DEBUG_NNTP) && verbose > 1)
+			debug_print_file("NNTP", "unparsable \"LIST COUNTS\" line: \"%s\"", line);
 #	endif /* DEBUG */
-		error_message(2, _(txt_unparseable_counts), line);
 		return FALSE;
 	}
 
@@ -306,7 +339,7 @@ parse_count_line(
  * We can't know the 'moderator' status and always return 'y'
  * But we don't change if the 'moderator' status is already checked by
  * read_active_file()
- * Returnes TRUE if NNTP is enabled and authentication is needed
+ * Returns TRUE if NNTP is enabled and authentication is needed
  */
 #ifdef NNTP_ABLE
 static t_bool
@@ -320,18 +353,16 @@ do_read_newsrc_active_file(
 	char *p;
 	char moderated[PATH_LEN];
 	int window = 0;
+	long processed = 0L;
 	t_artnum count = T_ARTNUM_CONST(-1), min = T_ARTNUM_CONST(1), max = T_ARTNUM_CONST(0);
-	t_artnum processed = T_ARTNUM_CONST(0);
 	static char ngname[NNTP_GRPLEN + 1]; /* RFC 3977 3.1 limits group names to 497 octets */
 	struct t_group *grpptr;
 #ifdef NNTP_ABLE
 	t_bool need_auth = FALSE;
-	char *ngnames[NUM_SIMULTANEOUS_GROUP_COMMAND];
-	int index_i = 0;
-	int index_o = 0;
+	char *ngnames[NUM_SIMULTANEOUS_GROUP_COMMAND] = { NULL };
+	int index_i = 0, index_o = 0;
 #endif /* NNTP_ABLE */
 
-
 	rewind(fp);
 
 	if (!batch_mode || verbose)
@@ -363,7 +394,7 @@ do_read_newsrc_active_file(
 				ngnames[index_i] = my_strdup(ptr);
 				snprintf(buf, sizeof(buf), "GROUP %s", ngnames[index_i]);
 #	ifdef DEBUG
-				if (debug & DEBUG_NNTP)
+				if ((debug & DEBUG_NNTP) && verbose > 1)
 					debug_print_file("NNTP", "read_newsrc_active_file() %s", buf);
 #	endif /* DEBUG */
 				put_server(buf);
@@ -382,10 +413,11 @@ do_read_newsrc_active_file(
 					 */
 					int i;
 					int j = index_o;
+
 					for (i = 0; i < window - 1; i++) {
 						snprintf(buf, sizeof(buf), "GROUP %s", ngnames[j]);
 #	ifdef DEBUG
-						if (debug & DEBUG_NNTP)
+						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "read_newsrc_active_file() %s", buf);
 #	endif /* DEBUG */
 						put_server(buf);
@@ -407,17 +439,12 @@ do_read_newsrc_active_file(
 
 							snprintf(fmt, sizeof(fmt), "%%"T_ARTNUM_SFMT" %%"T_ARTNUM_SFMT" %%"T_ARTNUM_SFMT" %%%ds", NNTP_GRPLEN);
 							if (sscanf(line, fmt, &count, &min, &max, ngname) != 4) {
-								error_message(2, _(txt_error_invalid_response_to_group), line);
 #	ifdef DEBUG
-								if (debug & DEBUG_NNTP) /* TODO: -> lang.c */
+								if ((debug & DEBUG_NNTP) && verbose > 1) {
 									debug_print_file("NNTP", "Invalid response to \"GROUP %s\": \"%s\"", ngnames[index_o], line);
-#	endif /* DEBUG */
-							}
-							if (strcmp(ngname, ngnames[index_o]) != 0) {
-								error_message(2, _(txt_error_wrong_newsgroupname_in_group_response), ngname, ngnames[index_o], line);
-#	ifdef DEBUG
-								if (debug & DEBUG_NNTP) /* TODO: -> lang.c */
-									debug_print_file("NNTP", "Groupname mismatch in response to \"GROUP %s\": \"%s\"", ngnames[index_o], line);
+									if (strcmp(ngname, ngnames[index_o]) != 0)
+										debug_print_file("NNTP", "Groupname mismatch in response to \"GROUP %s\": \"%s\"", ngnames[index_o], line);
+								}
 #	endif /* DEBUG */
 							}
 							ptr = ngname;
@@ -446,7 +473,7 @@ do_read_newsrc_active_file(
 
 					default:
 #	ifdef DEBUG
-						if (debug & DEBUG_NNTP)
+						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "NOT_OK %s", line);
 #	endif /* DEBUG */
 						free(ngnames[index_o]);
@@ -503,6 +530,7 @@ do_read_newsrc_active_file(
 #endif /* NNTP_ABLE */
 }
 
+
 /*
  * Wrapper for do_read_newsrc_active_file() to handle
  * missing authentication
@@ -536,9 +564,13 @@ read_newsrc_active_file(
 
 #ifdef NNTP_ABLE
 	if (need_auth) { /* delayed auth */
-		if (!authenticate(nntp_server, userid, FALSE) || do_read_newsrc_active_file(fp)) {
+		if (!authenticate(nntp_server, userid, FALSE))
+			tin_done(EXIT_FAILURE, _(txt_auth_failed), ERR_ACCESS);
+#	if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
+		set_maxartnum(FALSE);
+#	endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
+		if (do_read_newsrc_active_file(fp))
 			tin_done(EXIT_FAILURE, _(txt_auth_failed), ERR_ACCESS);
-		}
 	}
 #endif /* NNTP_ABLE */
 
@@ -584,41 +616,44 @@ read_active_file(
 	FILE *fp;
 	char *ptr;
 	char moderated[PATH_LEN];
-	t_artnum count = T_ARTNUM_CONST(-1), min = T_ARTNUM_CONST(1), max = T_ARTNUM_CONST(0);
 	long processed = 0L;
 	struct t_group *grpptr;
+	t_artnum count = T_ARTNUM_CONST(-1), min = T_ARTNUM_CONST(1), max = T_ARTNUM_CONST(0);
 
 	if (!batch_mode || verbose)
 		wait_message(0, _(txt_reading_news_active_file));
 
 	if ((fp = open_news_active_fp()) == NULL) {
-		if (cmd_line && !batch_mode)
+		if ((cmd_line && !batch_mode) || verbose)
 			my_fputc('\n', stderr);
 
 #ifdef NNTP_ABLE
 		if (read_news_via_nntp)
 			tin_done(EXIT_FAILURE, _(txt_cannot_retrieve), ACTIVE_FILE);
 #	ifndef NNTP_ONLY
-		else
+		else {
+			perror_message("%s", news_active_file);
 			tin_done(EXIT_FAILURE, _(txt_cannot_open_active_file), news_active_file, tin_progname);
+		}
 #	endif /* !NNTP_ONLY */
 #else
+		perror_message("%s", news_active_file);
 		tin_done(EXIT_FAILURE, _(txt_cannot_open), news_active_file);
 #endif /* NNTP_ABLE */
 	}
 
 	while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
 #if defined(DEBUG) && defined(NNTP_ABLE)
-		if (debug & DEBUG_NNTP)
+		if ((debug & DEBUG_NNTP) && verbose) /* long multiline response */
 			debug_print_file("NNTP", "<<<%s%s", logtime(), ptr);
 #endif /* DEBUG && NNTP_ABLE */
 
-		if (!parse_active_line(ptr, &max, &min, moderated))
-			continue;
-
 		if (++processed % MODULO_COUNT_NUM == 0)
 			spin_cursor();
 
+		if (!parse_active_line(ptr, &max, &min, moderated))
+			continue;
+
 		/*
 		 * Load group into group hash table
 		 * NULL means group already present, so we just fixup the counters
@@ -646,6 +681,10 @@ read_active_file(
 		 */
 		active_add(grpptr, count, max, min, moderated);
 	}
+#	if defined(DEBUG) && defined(NNTP_ABLE)
+	if ((debug & DEBUG_NNTP) && !verbose)
+		debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
+#	endif /* DEBUG && NNTP_ABLE */
 
 	TIN_FCLOSE(fp);
 
@@ -671,9 +710,9 @@ read_active_counts(
 	FILE *fp;
 	char *ptr;
 	char moderated[PATH_LEN];
-	t_artnum count = T_ARTNUM_CONST(-1), min = T_ARTNUM_CONST(1), max = T_ARTNUM_CONST(0);
-	t_artnum processed = T_ARTNUM_CONST(0);
+	long processed = 0L;
 	struct t_group *grpptr;
+	t_artnum count = T_ARTNUM_CONST(-1), min = T_ARTNUM_CONST(1), max = T_ARTNUM_CONST(0);
 
 	if (!batch_mode || verbose)
 		wait_message(0, _(txt_reading_news_active_file));
@@ -682,21 +721,21 @@ read_active_counts(
 		if (cmd_line && !batch_mode)
 			my_fputc('\n', stderr);
 
-		tin_done(EXIT_FAILURE,_(txt_cannot_retrieve), ACTIVE_FILE);
+		tin_done(EXIT_FAILURE, _(txt_cannot_retrieve), ACTIVE_FILE);
 	}
 
 	while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
 #	ifdef DEBUG
-		if (debug & DEBUG_NNTP)
+		if ((debug & DEBUG_NNTP) && verbose) /* long multiline response */
 			debug_print_file("NNTP", "<<<%s%s", logtime(), ptr);
 #	endif /* DEBUG */
 
-		if (!parse_count_line(ptr, &max, &min, &count, moderated))
-			continue;
-
 		if (++processed % MODULO_COUNT_NUM == 0)
 			spin_cursor();
 
+		if (!parse_count_line(ptr, &max, &min, &count, moderated))
+			continue;
+
 		/*
 		 * Load group into group hash table
 		 * NULL means group already present, so we just fixup the counters
@@ -724,6 +763,11 @@ read_active_counts(
 		 */
 		active_add(grpptr, count, max, min, moderated);
 	}
+#	ifdef DEBUG
+	/* log end of multiline response to get timing data */
+	if ((debug & DEBUG_NNTP) && !verbose)
+		debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
+#	endif /* DEBUG */
 
 	/*
 	 * Exit if active file wasn't read correctly or is empty
@@ -748,9 +792,9 @@ read_news_active_file(
 	FILE *fp;
 	int newgrps = 0;
 	t_bool do_group_cmds = !nntp_caps.list_counts;
-#ifdef NNTP_ABLE
+#if defined(NNTP_ABLE) && !defined(DISABLE_PIPELINING)
 	t_bool did_list_cmd = FALSE;
-#endif /* NNTP_ABLE */
+#endif /* NNTP_ABLE && !DISABLE_PIPELINING */
 
 	/*
 	 * Ignore -n if no .newsrc can be found or .newsrc is empty
@@ -771,7 +815,9 @@ read_news_active_file(
 	/* Read an active file if it is allowed */
 	if (list_active) {
 #ifdef NNTP_ABLE
+#	ifndef DISABLE_PIPELINING
 		did_list_cmd = TRUE;
+#	endif /* !DISABLE_PIPELINING */
 		if (read_news_via_nntp && nntp_caps.list_counts)
 			read_active_counts();
 		else
@@ -784,7 +830,7 @@ read_news_active_file(
 #ifdef NNTP_ABLE
 #	ifndef DISABLE_PIPELINING
 		/*
-		 * prefer LIST COUNTS, otherwise use LIST ACIVE (-l) or GROUP (-n)
+		 * prefer LIST COUNTS, otherwise use LIST ACTIVE (-l) or GROUP (-n)
 		 * or both (-ln); LIST COUNTS/ACTIVE grplist is used up to
 		 * PIPELINE_LIMIT groups in newsrc
 		 */
@@ -794,12 +840,12 @@ read_news_active_file(
 			char moderated[PATH_LEN];
 			int r = 0, j = 0;
 			int i;
-			t_artnum count = T_ARTNUM_CONST(-1), min = T_ARTNUM_CONST(1), max = T_ARTNUM_CONST(0);
 			struct t_group *grpptr;
+			t_artnum count = T_ARTNUM_CONST(-1), min = T_ARTNUM_CONST(1), max = T_ARTNUM_CONST(0);
 			t_bool need_auth = FALSE;
 
 			*buff = '\0';
-			/* we can't use for_each_group(i) yet, so we have to prase the newsrc */
+			/* we can't use for_each_group(i) yet, so we have to parse the newsrc */
 			if ((fp = fopen(newsrc, "r")) != NULL) {
 				while (tin_fgets(fp, FALSE) != NULL)
 					j++;
@@ -831,9 +877,9 @@ read_news_active_file(
 						put_server(buff);
 						r++;
 					}
-				} else {
+				} else
 					do_group_cmds = TRUE;
-				}
+
 				fclose(fp);
 
 				if (j < PIPELINE_LIMIT) {
@@ -842,15 +888,15 @@ read_news_active_file(
 							/* TODO: add 483 (RFC 3977) code */
 							if (j == ERR_NOAUTH || j == NEED_AUTHINFO)
 								need_auth = TRUE;
-#if 0 /* do we need something like this? */
+#		if 0 /* do we need something like this? */
 							if (j == ERR_CMDSYN)
 								list_active = TRUE;
-#endif /* 0 */
+#		endif /* 0 */
 							continue;
 						} else {
 							while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
 #		ifdef DEBUG
-								if (debug & DEBUG_NNTP)
+								if ((debug & DEBUG_NNTP) && verbose) /* long multiline response */
 									debug_print_file("NNTP", "<<<%s%s", logtime(), ptr);
 #		endif /* DEBUG */
 								if (nntp_caps.type == CAPABILITIES && nntp_caps.list_counts) {
@@ -877,11 +923,17 @@ read_news_active_file(
 								}
 								active_add(grpptr, count, max, min, moderated);
 							}
+#	ifdef DEBUG
+							/* log end of multiline response to get timing data */
+							if ((debug & DEBUG_NNTP) && !verbose)
+								debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
+#	endif /* DEBUG */
 						}
 					}
 					if (need_auth) { /* retry after auth is overkill here, so just auth */
 						if (!authenticate(nntp_server, userid, FALSE))
 							tin_done(EXIT_FAILURE, _(txt_auth_failed), nntp_caps.type == CAPABILITIES ? ERR_AUTHFAIL : ERR_ACCESS);
+						/* no set_maxartnum() here after auth as we're pipelining ... */
 					}
 				}
 				did_reconnect = FALSE;
@@ -903,9 +955,11 @@ read_news_active_file(
 		newgrps = check_for_any_new_groups();
 
 	/*
-	 * finally we have a list of all groups an can set the attributes
+	 * finally we have a list of all groups and can set the attributes
 	 */
+	BegStopWatch();
 	assign_attributes_to_groups();
+	EndStopWatch("assign_attributes_to_groups()");
 
 	return newgrps;
 }
@@ -926,14 +980,14 @@ open_newgroups_fp(
 	if (read_news_via_nntp && !read_saved_news) {
 		/*
 		 * not checking for caps_type == CAPABILITIES && reader as some
-		 * servers do not support it even if advertizing READER so we must
+		 * servers do not support it even if advertising READER so we must
 		 * handle errors anyway and just issue the cmd.
 		 */
 		if (idx == -1 || ((ngtm = localtime(&newnews[idx].time)) == NULL))
 			return (FILE *) 0;
 
 		/*
-		 * RFC 3077 states that we SHOULD use 4 digit year but some servers
+		 * RFC 3977 states that we SHOULD use 4 digit year but some servers
 		 * still do not support it.
 		 */
 		snprintf(line, sizeof(line), "NEWGROUPS %02d%02d%02d %02d%02d%02d",
@@ -942,6 +996,9 @@ open_newgroups_fp(
 
 		return (nntp_command(line, OK_NEWGROUPS, NULL, 0));
 	}
+#else
+	/* silence compiler warning (unused parameter) */
+	(void) idx;
 #endif /* NNTP_ABLE */
 	return (fopen(active_times_file, "r"));
 }
@@ -986,7 +1043,7 @@ check_for_any_new_groups(
 	}
 
 #ifdef DEBUG
-	if (debug & DEBUG_NNTP)
+	if ((debug & DEBUG_NNTP) && verbose > 1)
 		debug_print_file("NNTP", "Newnews old=[%lu]  new=[%lu]", (unsigned long int) old_newnews_time, (unsigned long int) new_newnews_time);
 #endif /* DEBUG */
 
@@ -1087,7 +1144,7 @@ subscribe_new_group(
 		/*
 		 * as subscribe_new_group() is called from check_for_any_new_groups()
 		 * which has pending data on the socket if reading via NNTP we are not
-		 * allowed to issue any NNTP comands yet
+		 * allowed to issue any NNTP commands yet
 		 */
 		subscribe(&active[my_group[idx]], SUBSCRIBED, bool_not(read_news_via_nntp));
 		/*
@@ -1127,7 +1184,7 @@ match_group_list(
 		separator = strchr(group_list, ',');
 		group_len = MIN(((separator == NULL) ? list_len : (size_t) (separator - group_list)), sizeof(pattern) - 1);
 
-		if ((negate = ('!' == *group_list))) {
+		if ((negate = (*group_list == '!'))) {
 			/*
 			 * a '!' before the pattern inverts sense of match
 			 */
@@ -1149,7 +1206,7 @@ match_group_list(
 		/*
 		 * now examine next entry if any
 		 */
-		if ((char) 0 != group_list[group_len])
+		if (group_list[group_len] != '\0')
 			group_len++;	/* skip the separator */
 
 		group_list += group_len;
@@ -1208,7 +1265,7 @@ load_newnews_info(
 	newnews[i].time = new_time;
 
 #ifdef DEBUG
-	if (debug & DEBUG_NNTP)
+	if ((debug & DEBUG_NNTP) && verbose > 1)
 		debug_print_file("NNTP", "ACTIVE host=[%s] time=[%lu]", newnews[i].host, (unsigned long int) newnews[i].time);
 #endif /* DEBUG */
 }
@@ -1258,7 +1315,6 @@ group_flag(
 }
 
 
-/* ex actived.c functions */
 void
 create_save_active_file(
 	void)
@@ -1272,7 +1328,7 @@ create_save_active_file(
 	if (no_write && file_size(local_save_active_file) != -1L)
 		return;
 
-	if (strfpath(cmdline.args & CMDLINE_SAVEDIR ? cmdline.savedir : tinrc.savedir, group_path, sizeof(group_path), NULL, FALSE)) {
+	if (strfpath((cmdline.args & CMDLINE_SAVEDIR) ? cmdline.savedir : tinrc.savedir, group_path, sizeof(group_path), NULL, FALSE)) {
 		wait_message(0, _(txt_creating_active));
 		print_active_head(local_save_active_file);
 
@@ -1280,7 +1336,7 @@ create_save_active_file(
 			group_path[strlen(group_path) - 1] = '\0';
 
 		fb = my_strdup(group_path);
-		make_group_list(local_save_active_file, cmdline.args & CMDLINE_SAVEDIR ? cmdline.savedir : tinrc.savedir, fb, group_path);
+		make_group_list(local_save_active_file, (cmdline.args & CMDLINE_SAVEDIR) ? cmdline.savedir : tinrc.savedir, fb, group_path);
 		free(fb);
 	}
 }
@@ -1322,7 +1378,7 @@ make_group_list(
 				make_group_list(active_file, base_dir, fixed_base, group_path);
 				find_art_max_min(group_path, &art_max, &art_min);
 				append_group_line(active_file, group_path + strlen(fixed_base) + 1, art_max, art_min, fixed_base);
-				if ((ptr = strrchr(group_path, '/')) != NULL) /* TODO: Unix'ism */
+				if ((ptr = strrchr(group_path, '/')) != NULL)
 					*ptr = '\0';
 			}
 		}
@@ -1369,11 +1425,7 @@ append_group_line(
 				clearerr(fp);
 				fclose(fp);
 			}
-			err = rename(file_tmp, active_file);
-#ifdef DEBUG
-			if ((debug & DEBUG_MISC) && err) /* TODO: is this the right debug-level? */
-				perror_message(_(txt_rename_error), file_tmp, active_file);
-#endif /* DEBUG */
+			rename_file(file_tmp, active_file);
 		}
 		free(group_name);
 	}
